@@ -4,7 +4,6 @@ __docformat__ = "restructuredtext en"
 
 __all__ = ['dok_matrix', 'isspmatrix_dok']
 
-import operator
 from itertools import izip
 
 import numpy as np
@@ -12,13 +11,21 @@ import numpy as np
 from base import spmatrix, isspmatrix
 from sputils import isdense, getdtype, isshape, isintlike, isscalarlike, upcast
 
+try:
+    from operator import isSequenceType as _is_sequence
+except ImportError:
+    def _is_sequence(x):
+        return (hasattr(x, '__len__') or hasattr(x, '__next__')
+                or hasattr(x, 'next'))
+
 class dok_matrix(spmatrix, dict):
-    """Dictionary Of Keys based sparse matrix.
+    """
+    Dictionary Of Keys based sparse matrix.
 
     This is an efficient structure for constructing sparse
     matrices incrementally.
 
-    This can be instatiated in several ways:
+    This can be instantiated in several ways:
         dok_matrix(D)
             with a dense matrix, D
 
@@ -136,7 +143,7 @@ class dok_matrix(spmatrix, dict):
             if isinstance(i, slice):
                 # Is there an easier way to do this?
                 seq = xrange(i.start or 0, i.stop or self.shape[0], i.step or 1)
-            elif operator.isSequenceType(i):
+            elif _is_sequence(i):
                 seq = i
             else:
                 # Make sure i is an integer. (But allow it to be a subclass of int).
@@ -167,26 +174,26 @@ class dok_matrix(spmatrix, dict):
                     ###################################
                     # We should reshape the new matrix here!
                     ###################################
-                    raise NotImplementedError, "fancy indexing supported over" \
-                            " one axis only"
+                    raise NotImplementedError("fancy indexing supported over"
+                            " one axis only")
                 return new
 
             # Below here, j is a sequence, but i is an integer
             if isinstance(j, slice):
                 # Is there an easier way to do this?
                 seq = xrange(j.start or 0, j.stop or self.shape[1], j.step or 1)
-            elif operator.isSequenceType(j):
+            elif _is_sequence(j):
                 seq = j
             else:
                 # j is not an integer
-                raise TypeError, "index must be a pair of integers or slices"
+                raise TypeError("index must be a pair of integers or slices")
 
             # Create a new matrix of the correct dimensions
             first = seq[0]
             last = seq[-1]
             if first < 0 or first >= self.shape[1] or last < 0 \
                          or last >= self.shape[1]:
-                raise IndexError, "index out of bounds"
+                raise IndexError("index out of bounds")
             newshape = (1, last-first+1)
             new = dok_matrix(newshape)
             # ** This uses linear time in the size n of dimension 1:
@@ -205,7 +212,7 @@ class dok_matrix(spmatrix, dict):
         try:
             i, j = key
         except (ValueError, TypeError):
-            raise TypeError, "index must be a pair of integers or slices"
+            raise TypeError("index must be a pair of integers or slices")
 
         # First deal with the case where both i and j are integers
         if isintlike(i) and isintlike(j):
@@ -215,27 +222,29 @@ class dok_matrix(spmatrix, dict):
                 j += self.shape[1]
 
             if i < 0 or i >= self.shape[0] or j < 0 or j >= self.shape[1]:
-                raise IndexError, "index out of bounds"
+                raise IndexError("index out of bounds")
 
             if np.isscalar(value):
-                if value==0:
-                    del self[(i,j)]
+                if value == 0:
+                    if self.has_key((i,j)):
+                        del self[(i,j)]
                 else:
                     dict.__setitem__(self, (i,j), self.dtype.type(value))
             else:
-                raise TypeError, "cannot set matrix element to a non-scalar"
+                raise ValueError('setting an array element with a sequence')
+
         else:
             # Either i or j is a slice, sequence, or invalid.  If i is a slice
             # or sequence, unfold it first and call __setitem__ recursively.
             if isinstance(i, slice):
                 # Is there an easier way to do this?
                 seq = xrange(i.start or 0, i.stop or self.shape[0], i.step or 1)
-            elif operator.isSequenceType(i):
+            elif _is_sequence(i):
                 seq = i
             else:
                 # Make sure i is an integer. (But allow it to be a subclass of int).
                 if not isintlike(i):
-                    raise TypeError, "index must be a pair of integers or slices"
+                    raise TypeError("index must be a pair of integers or slices")
                 seq = None
             if seq is not None:
                 # First see if 'value' is another dok_matrix of the appropriate
@@ -245,8 +254,8 @@ class dok_matrix(spmatrix, dict):
                         for element in seq:
                             self[element, j] = value[element, 0]
                     else:
-                        raise NotImplementedError, "setting a 2-d slice of" \
-                                " a dok_matrix is not yet supported"
+                        raise NotImplementedError("setting a 2-d slice of"
+                                " a dok_matrix is not yet supported")
                 elif np.isscalar(value):
                     for element in seq:
                         self[element, j] = value
@@ -254,12 +263,12 @@ class dok_matrix(spmatrix, dict):
                     # See if value is a sequence
                     try:
                         if len(seq) != len(value):
-                            raise ValueError, "index and value ranges must" \
-                                              " have the same length"
+                            raise ValueError("index and value ranges must"
+                                              " have the same length")
                     except TypeError:
                         # Not a sequence
-                        raise TypeError, "unsupported type for" \
-                                         " dok_matrix.__setitem__"
+                        raise TypeError("unsupported type for"
+                                         " dok_matrix.__setitem__")
 
                     # Value is a sequence
                     for element, val in izip(seq, value):
@@ -271,11 +280,11 @@ class dok_matrix(spmatrix, dict):
                 # Process j
                 if isinstance(j, slice):
                     seq = xrange(j.start or 0, j.stop or self.shape[1], j.step or 1)
-                elif operator.isSequenceType(j):
+                elif _is_sequence(j):
                     seq = j
                 else:
                     # j is not an integer
-                    raise TypeError, "index must be a pair of integers or slices"
+                    raise TypeError("index must be a pair of integers or slices")
 
                 # First see if 'value' is another dok_matrix of the appropriate
                 # dimensions
@@ -284,8 +293,8 @@ class dok_matrix(spmatrix, dict):
                         for element in seq:
                             self[i, element] = value[0, element]
                     else:
-                        raise NotImplementedError, "setting a 2-d slice of" \
-                                " a dok_matrix is not yet supported"
+                        raise NotImplementedError("setting a 2-d slice of"
+                                " a dok_matrix is not yet supported")
                 elif np.isscalar(value):
                     for element in seq:
                         self[i, element] = value
@@ -293,11 +302,11 @@ class dok_matrix(spmatrix, dict):
                     # See if value is a sequence
                     try:
                         if len(seq) != len(value):
-                            raise ValueError, "index and value ranges must have" \
-                                              " the same length"
+                            raise ValueError("index and value ranges must have"
+                                              " the same length")
                     except TypeError:
                         # Not a sequence
-                        raise TypeError, "unsupported type for dok_matrix.__setitem__"
+                        raise TypeError("unsupported type for dok_matrix.__setitem__")
                     else:
                         for element, val in izip(seq, value):
                             self[i, element] = val
@@ -317,7 +326,7 @@ class dok_matrix(spmatrix, dict):
             #new.dtype.char = self.dtype.char
         elif isinstance(other, dok_matrix):
             if other.shape != self.shape:
-                raise ValueError, "matrix dimensions are not equal"
+                raise ValueError("matrix dimensions are not equal")
             # We could alternatively set the dimensions to the the largest of
             # the two matrices to be summed.  Would this be a good idea?
             new = dok_matrix(self.shape, dtype=self.dtype)
@@ -330,7 +339,7 @@ class dok_matrix(spmatrix, dict):
         elif isdense(other):
             new = self.todense() + other
         else:
-            raise TypeError, "data type not understood"
+            raise TypeError("data type not understood")
         return new
 
     def __radd__(self, other):
@@ -346,7 +355,7 @@ class dok_matrix(spmatrix, dict):
                         new[i, j] = aij
         elif isinstance(other, dok_matrix):
             if other.shape != self.shape:
-                raise ValueError, "matrix dimensions are not equal"
+                raise ValueError("matrix dimensions are not equal")
             new = dok_matrix(self.shape, dtype=self.dtype)
             new.update(self)
             for key in other:
@@ -357,7 +366,7 @@ class dok_matrix(spmatrix, dict):
         elif isdense(other):
             new = other + self.todense()
         else:
-            raise TypeError, "data type not understood"
+            raise TypeError("data type not understood")
         return new
 
     def __neg__(self):
@@ -440,7 +449,7 @@ class dok_matrix(spmatrix, dict):
         M, N = self.shape
         new = dok_matrix((N, M), dtype=self.dtype)
         for key, value in self.iteritems():
-            new[key[1], key[0]] = conj(value)
+            new[key[1], key[0]] = np.conj(value)
         return new
 
     def copy(self):
@@ -456,13 +465,13 @@ class dok_matrix(spmatrix, dict):
         N = len(cols_or_rows)
         if indx: # columns
             for key in self.keys():
-                num = searchsorted(cols_or_rows, key[1])
+                num = np.searchsorted(cols_or_rows, key[1])
                 if num < N:
                     newkey = (key[0], num)
                     new[newkey] = self[key]
         else:
             for key in self.keys():
-                num = searchsorted(cols_or_rows, key[0])
+                num = np.searchsorted(cols_or_rows, key[0])
                 if num < N:
                     newkey = (num, key[1])
                     new[newkey] = self[key]
@@ -476,7 +485,7 @@ class dok_matrix(spmatrix, dict):
         indx = int((columns == 1))
         if indx:
             for key in self.keys():
-                num = searchsorted(cols_or_rows, key[1])
+                num = np.searchsorted(cols_or_rows, key[1])
                 if cols_or_rows[num] == key[1]:
                     newkey = (key[0], num)
                     ext[newkey] = self[key]
@@ -485,7 +494,7 @@ class dok_matrix(spmatrix, dict):
                     base[newkey] = self[key]
         else:
             for key in self.keys():
-                num = searchsorted(cols_or_rows, key[0])
+                num = np.searchsorted(cols_or_rows, key[0])
                 if cols_or_rows[num] == key[0]:
                     newkey = (num, key[1])
                     ext[newkey] = self[key]
@@ -522,12 +531,13 @@ class dok_matrix(spmatrix, dict):
         return self.tocoo().toarray()
 
     def resize(self, shape):
-        """ Resize the matrix to dimensions given by 'shape', removing any
-        non-zero elements that lie outside.
+        """ Resize the matrix in-place to dimensions given by 'shape'.
+        
+        Any non-zero elements that lie outside the new shape are removed.
         """
         if not isshape(shape):
-            raise TypeError, "dimensions must be a 2-tuple of positive"\
-                             " integers"
+            raise TypeError("dimensions must be a 2-tuple of positive"
+                             " integers")
         newM, newN = shape
         M, N = self.shape
         if newM < M or newN < N:
@@ -535,7 +545,7 @@ class dok_matrix(spmatrix, dict):
             for (i, j) in self.keys():
                 if i >= newM or j >= newN:
                     del self[i, j]
-        self.shape = shape
+        self._shape = shape
 
 
 
