@@ -3,6 +3,8 @@
 #from info import __doc__
 from iterative import *
 from minres import minres
+from lgmres import lgmres
+from lsqr import lsqr
 
 __all__ = filter(lambda s:not s.startswith('_'),dir())
 from numpy.testing import Tester
